@@ -7,11 +7,11 @@ SporeStack API request/response models
 
 from datetime import datetime
 from enum import Enum
-from typing import Dict, List, Optional
+from typing import Dict, List, Optional, Union
 
 from pydantic import BaseModel, Field
 
-from .models import Flavor, Payment
+from .models import Flavor, OperatingSystem, Payment
 
 
 class TokenAdd:
@@ -85,7 +85,7 @@ class ServerTopup:
 
     class Request(BaseModel):
         days: int
-        token: str
+        token: Union[str, None] = None
 
 
 class ServerInfo:
@@ -170,7 +170,7 @@ class OperatingSystems:
     method = "GET"
 
     class Response(BaseModel):
-        operating_systems: List[str]
+        operating_systems: Dict[str, OperatingSystem]
 
 
 class TokenMessageSender(str, Enum):
