@@ -30,7 +30,7 @@ _home = os.getenv("HOME", None)
 assert _home is not None, "Unable to detect $HOME environment variable?"
 HOME = Path(_home)
 
-SPORESTACK_DIR = HOME / ".sporestack"
+SPORESTACK_DIR = Path(os.getenv("SPORESTACK_DIR", HOME / ".sporestack"))
 
 # Try to protect files in ~/.sporestack
 os.umask(0o0077)
@@ -44,7 +44,7 @@ cli.add_typer(token_cli, name="token")
 server_cli = typer.Typer(help="Commands to interact with SporeStack servers")
 cli.add_typer(server_cli, name="server")
 
-_log_level = os.getenv("LOG_LEVEL", "info").upper()
+_log_level = os.getenv("LOG_LEVEL", "warning").upper()
 _numeric_log_level = getattr(logging, _log_level, None)
 if _numeric_log_level is None:
     raise ValueError(f"LOG_LEVEL: {_log_level} is invalid. Aborting!")
@@ -105,9 +105,7 @@ def launch(
         True, help="Wait for server to be assigned an IP address."
     ),
 ) -> None:
-    """
-    Launch a server on SporeStack.
-    """
+    """Launch a server on SporeStack."""
     typer.echo(f"Launching server with token {token}...", err=True)
     _token = load_token(token)
 
@@ -185,9 +183,7 @@ def topup(
     days: int = typer.Option(...),
     token: str = DEFAULT_TOKEN,
 ) -> None:
-    """
-    Extend an existing SporeStack server's lifetime.
-    """
+    """Extend an existing SporeStack server's lifetime."""
 
     from .api_client import APIClient
 
@@ -211,7 +207,11 @@ def server_info_path() -> Path:
     servers_dir = SPORESTACK_DIR / "servers"
 
     # Migrate existing server.json files into servers subdirectory
-    if SPORESTACK_DIR.exists() and not servers_dir.exists():
+    if (
+        SPORESTACK_DIR.exists()
+        and not servers_dir.exists()
+        and len(list(SPORESTACK_DIR.glob("*.json"))) > 0
+    ):
         typer.echo(
             f"Migrating server profiles found in {SPORESTACK_DIR} to {servers_dir}.",
             err=True,
@@ -318,9 +318,7 @@ def server_list(
         True, help="List older servers not associated to token."
     ),
 ) -> None:
-    """
-    List all locally known servers and all servers under the given token.
-    """
+    """List all locally known servers and all servers under the given token."""
     from .api_client import APIClient
     from .exceptions import SporeStackUserError
 
@@ -419,11 +417,11 @@ def _get_machine_id(machine_id: str, hostname: str, token: str) -> str:
     raise typer.Exit(code=1)
 
 
-@server_cli.command()
-def info(hostname: str = "", machine_id: str = "", token: str = DEFAULT_TOKEN) -> None:
-    """
-    Info on the VM
-    """
+@server_cli.command(name="info")
+def server_info(
+    hostname: str = "", machine_id: str = "", token: str = DEFAULT_TOKEN
+) -> None:
+    """Show information about the server."""
     machine_id = _get_machine_id(machine_id=machine_id, hostname=hostname, token=token)
     from .api_client import APIClient
 
@@ -435,9 +433,7 @@ def info(hostname: str = "", machine_id: str = "", token: str = DEFAULT_TOKEN) -
 def server_info_json(
     hostname: str = "", machine_id: str = "", token: str = DEFAULT_TOKEN
 ) -> None:
-    """
-    Info on the VM, in JSON format
-    """
+    """Info on the server, in JSON format."""
     machine_id = _get_machine_id(machine_id=machine_id, hostname=hostname, token=token)
     from .api_client import APIClient
 
@@ -460,9 +456,7 @@ def start(hostname: str = "", machine_id: str = "", token: str = DEFAULT_TOKEN)
 
 @server_cli.command()
 def stop(hostname: str = "", machine_id: str = "", token: str = DEFAULT_TOKEN) -> None:
-    """
-    Immediately shuts down the VM.
-    """
+    """Power off the server. (Not a graceful shutdown)"""
     machine_id = _get_machine_id(machine_id=machine_id, hostname=hostname, token=token)
     from .api_client import APIClient
 
@@ -475,9 +469,7 @@ def stop(hostname: str = "", machine_id: str = "", token: str = DEFAULT_TOKEN) -
 def autorenew_enable(
     hostname: str = "", machine_id: str = "", token: str = DEFAULT_TOKEN
 ) -> None:
-    """
-    Enable autorenew on a server.
-    """
+    """Enable autorenew on a server."""
     machine_id = _get_machine_id(machine_id=machine_id, hostname=hostname, token=token)
     from .api_client import APIClient
 
@@ -505,9 +497,7 @@ def autorenew_disable(
 def delete(
     hostname: str = "", machine_id: str = "", token: str = DEFAULT_TOKEN
 ) -> None:
-    """
-    Delete the VM before its expiration
-    """
+    """Delete the server before its expiration."""
     machine_id = _get_machine_id(machine_id=machine_id, hostname=hostname, token=token)
     from .api_client import APIClient
 
@@ -522,9 +512,7 @@ def delete(
 def forget(
     hostname: str = "", machine_id: str = "", token: str = DEFAULT_TOKEN
 ) -> None:
-    """
-    Forget about a deleted server so that it doesn't show up in server list.
-    """
+    """Forget about a deleted server so that it doesn't show up in server list."""
     machine_id = _get_machine_id(machine_id=machine_id, hostname=hostname, token=token)
     from .api_client import APIClient
 
@@ -552,9 +540,7 @@ def rebuild(
 
 @server_cli.command()
 def flavors() -> None:
-    """
-    Returns available flavors.
-    """
+    """Show available flavors."""
     from .api_client import APIClient
 
     api_client = APIClient(api_endpoint=get_api_endpoint())
@@ -565,9 +551,7 @@ def flavors() -> None:
 
 @server_cli.command()
 def operating_systems() -> None:
-    """
-    Returns available operating systems.
-    """
+    """Show available operating systems."""
     from .api_client import APIClient
 
     api_client = APIClient(api_endpoint=get_api_endpoint())
@@ -612,7 +596,7 @@ def token_create(
     currency: str = typer.Option(...),
 ) -> None:
     """
-    Enables a new settlement token.
+    Enables a new token.
 
     Dollars is starting balance.
     """
@@ -627,6 +611,7 @@ def token_create(
         raise typer.Exit(1)
 
     from .api_client import APIClient
+    from .exceptions import SporeStackServerError
 
     api_client = APIClient(api_endpoint=get_api_endpoint())
 
@@ -634,7 +619,6 @@ def token_create(
         token=_token,
         dollars=dollars,
         currency=currency,
-        retry=True,
     )
 
     uri = response.payment.uri
@@ -650,12 +634,15 @@ def token_create(
         # FIXME: Wait two hours in a smarter way.
         # Waiting for payment to set in.
         time.sleep(10)
-        response = api_client.token_add(
-            token=_token,
-            dollars=dollars,
-            currency=currency,
-            retry=True,
-        )
+        try:
+            response = api_client.token_add(
+                token=_token,
+                dollars=dollars,
+                currency=currency,
+            )
+        except SporeStackServerError:
+            typer.echo("Received 500 HTTP status, will try again.", err=True)
+            continue
         if response.payment.paid is True:
             typer.echo(f"{token} has been enabled with ${dollars}.")
             typer.echo(f"{token}'s key is {_token}.")
@@ -670,9 +657,7 @@ def token_import(
     name: str = typer.Argument(DEFAULT_TOKEN),
     key: str = typer.Option(...),
 ) -> None:
-    """
-    Imports a token from a key.
-    """
+    """Imports a token under the given name."""
     save_token(name, key)
 
 
@@ -682,12 +667,11 @@ def token_topup(
     dollars: int = typer.Option(...),
     currency: str = typer.Option(...),
 ) -> None:
-    """
-    Adds balance to an existing settlement token.
-    """
+    """Adds balance to an existing token."""
     token = load_token(token)
 
     from .api_client import APIClient
+    from .exceptions import SporeStackServerError
 
     api_client = APIClient(api_endpoint=get_api_endpoint())
 
@@ -695,7 +679,6 @@ def token_topup(
         token,
         dollars,
         currency=currency,
-        retry=True,
     )
 
     uri = response.payment.uri
@@ -709,12 +692,15 @@ def token_topup(
         typer.echo(WAITING_PAYMENT_TO_PROCESS, err=True)
         tries = tries - 1
         # FIXME: Wait two hours in a smarter way.
-        response = api_client.token_add(
-            token,
-            dollars,
-            currency=currency,
-            retry=True,
-        )
+        try:
+            response = api_client.token_add(
+                token=token,
+                dollars=dollars,
+                currency=currency,
+            )
+        except SporeStackServerError:
+            typer.echo("Received 500 HTTP status, will try again.", err=True)
+            continue
         # Waiting for payment to set in.
         time.sleep(10)
         if response.payment.paid is True:
@@ -725,9 +711,7 @@ def token_topup(
 
 @token_cli.command()
 def balance(token: str = typer.Argument(DEFAULT_TOKEN)) -> None:
-    """
-    Gets balance for a settlement token.
-    """
+    """Shows a token's balance."""
     _token = load_token(token)
 
     from .api_client import APIClient
@@ -737,11 +721,30 @@ def balance(token: str = typer.Argument(DEFAULT_TOKEN)) -> None:
     typer.echo(api_client.token_balance(token=_token).usd)
 
 
+@token_cli.command(name="info")
+def token_info(token: str = typer.Argument(DEFAULT_TOKEN)) -> None:
+    """Show information about a token, including balance."""
+    _token = load_token(token)
+
+    from .api_client import APIClient
+    from .client import Client
+
+    api_client = APIClient(api_endpoint=get_api_endpoint())
+    client = Client(api_client=api_client, client_token=_token)
+
+    info = client.token.info()
+    typer.echo(f"Balance: {info.balance_usd} ({info.balance_cents} cents)")
+    typer.echo(f"Total servers: {info.servers}")
+    typer.echo(f"Burn rate: {info.burn_rate_usd} per day (of servers set to autorenew)")
+    typer.echo(
+        f"Days remaining: {info.days_remaining} (for servers set to autorenew, "
+        "given the remaining balance)"
+    )
+
+
 @token_cli.command()
 def servers(token: str = typer.Argument(DEFAULT_TOKEN)) -> None:
-    """
-    Returns server info for servers launched by a given token.
-    """
+    """Returns server info for servers launched by a given token."""
     _token = load_token(token)
 
     from .api_client import APIClient
@@ -753,9 +756,7 @@ def servers(token: str = typer.Argument(DEFAULT_TOKEN)) -> None:
 
 @token_cli.command(name="list")
 def token_list() -> None:
-    """
-    Gets balance for a settlement token.
-    """
+    """List tokens."""
     token_dir = token_path()
     typer.echo(f"SporeStack tokens present in {token_dir}:", err=True)
     typer.echo("(Name): (Key)", err=True)
@@ -767,9 +768,7 @@ def token_list() -> None:
 
 @token_cli.command()
 def messages(token: str = typer.Argument(DEFAULT_TOKEN)) -> None:
-    """
-    Show support messages.
-    """
+    """Show support messages."""
     token = load_token(token)
 
     from .api_client import APIClient
@@ -789,9 +788,7 @@ def messages(token: str = typer.Argument(DEFAULT_TOKEN)) -> None:
 def send_message(
     token: str = typer.Argument(DEFAULT_TOKEN), message: str = typer.Option(...)
 ) -> None:
-    """
-    Send a support message.
-    """
+    """Send a support message."""
     token = load_token(token)
 
     from .api_client import APIClient
@@ -805,9 +802,7 @@ def send_message(
 
 @cli.command()
 def version() -> None:
-    """
-    Returns the installed version.
-    """
+    """Returns the installed version."""
     from . import __version__
 
     typer.echo(__version__)
