@@ -8,6 +8,12 @@ class SporeStackUserError(SporeStackError):
     pass
 
 
+class SporeStackTooManyRequestsError(SporeStackError):
+    """HTTP 429, retry again later"""
+
+    pass
+
+
 class SporeStackServerError(SporeStackError):
     """HTTP 5XX"""
 
