@@ -39,3 +39,23 @@ class Flavor(BaseModel):
     ipv6: str
     # Gigabytes of bandwidth per day
     bandwidth: int
+
+
+class OperatingSystem(BaseModel):
+    slug: str
+    """Unique string to identify the operating system."""
+    minimum_disk: int
+    """Minimum disk storage required in GiB"""
+    provider_slug: str
+    """Unique string to identify the operating system."""
+
+
+class TokenInfo(BaseModel):
+    balance_cents: int
+    balance_usd: str
+    burn_rate: int
+    """Deprecated."""
+    burn_rate_cents: int
+    burn_rate_usd: str
+    days_remaining: int
+    servers: int
